/*
#include <OpenGL/gl.h>  // pour MacOSX
#include <OpenGL/glu.h> // pour MacOSX
#include <GLUT/glut.h>  // pour MacOSX
*/

#include <GL/glut.h> // pour PC
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#define NMAX 50  // nombre max de villes
#define HEIGHT 800 // dimension fenetre
#define WIDTH 600  // dimension fenetre

/*
Voyageur de commerce resolu avec:

Kohonen (visualisation et saisie des entrees en openGL)
Par Julien GONY & Didier STRAUS

Compilation sur MacOSX :
gcc -framework OpenGL -framework GLUT -framework Cocoa -o voyageur voyageur.c

pour linux
gcc voyageur.c -o voyageur -L/usr/X11R6/include/GL -L/usr/X11R6/lib -lGL -lGLU -lglut -lm -lX11

pour windows: dans l'editeur de lien -lopengl32 -lglu32 -lglut32
*/

/************* TYPE ***************/

typedef struct neurone{// pour representation graphique
	float x;
	float y;
}neurone;

/********** VAR GLOBALE *******************/

neurone tab[NMAX];// le tableau de poids
neurone entrees[NMAX]; // stockera les entrees (les villes).

char presse; // touche presse
int etape=1; // etape 1 -->saisir les coordonnees
             // etape 2 -->affiche le tableau des poids
            
int nbEntrees=0; // nombre d'entrees cres
int temps; // compte le nombre d'iterations d'apprentissage

float eta[7]={0.50,0.45,0.38,0.20,0.15,0.1,0.05};//fonction eta (par rapport au voisinage)

/************* Prototypes des fonctions d'affichage *****************/

void affichage();
void clavier(unsigned char touche,int x,int y);
void reshape(int w,int h);
void mouse(int bouton,int etat,int x,int y);

/************** Prototypes des fonctions pour Kohonen *****************/

float delta(int i0,int i); // coeficient pour le voisinage
void initPoids(); // initialisatin des poids en cercle
int nVainqueur(neurone n); // determination du neurone vainqueur
void apprentissagePoids(int i0,neurone n); // MAJ des poids pour le neuronne vainqueur i0
void reduireEta();//reduit  la fois l'influence du voisinage, et la valeur de eta.
void unApprentissage(); // une iteration pour chaque entrees.


/************** fonctions pour kohonen ********************/

float delta(int i0,int i) {
	int res,res1;
res=abs(i0-i);//distance de voisinage
if(i>i0){//Le voisinage est un anneaux (le dernier est pres du premier)
  res1=nbEntrees-i+i0;
  }
else{
  res1=nbEntrees-i0+i;
  }
if((res<res1)&&(res<7)){//plus petite distance (directe ou en faisant le tour)
  return(eta[res]);//valeur de eta
  }
else{
  if(res1<7){
    return(eta[res1]);//valeur de eta
    }
   }    

	return (0.0);
} /* delta */

/***********************************************************/

void reduireEta(){
  int i;
  float pas = 0.0005;//le pas de reduction
  if(eta[0]<=pas){//dernier pas (pour que les neurones ailles sur les villes)
    eta[0]=1;
  }
  else{
    eta[0]-=pas;//reduire
    }
  for(i=1;i<7;i++){//reduire eta quand c'est Superieur au pas
        if(eta[i]<=i*pas){
           eta[i]=0.;
        }
        else{
          if(eta[i]!=0.){
            eta[i]-=i*i*pas;//reduction non linaire
          }
        }
  }
 
}/* reduireEta */

/* **************************************************** */

void initPoids(){
	int i,j,n;
	float angle=(2*M_PI)/(nbEntrees-1);
	
	for(i=0;i<nbEntrees;i++) {//en cercle de rayaon 0.01
      tab[i].x=0.01*cos((2*M_PI)-i*angle);
      tab[i].y=0.01*sin((2*M_PI)-i*angle);
	}
      	
} /* initPoids */

/* **************************************************** */

int nVainqueur(neurone n){
	int i,i0;
	float d,dmax=1000;
	
	for(i=0;i<nbEntrees;i++){ 
		d=pow(n.x-tab[i].x,2)+pow(n.y-tab[i].y,2);//distance neurone avec vainqueur
		if (d<dmax) {
			dmax=d;
			i0=i;
		}
	}
	return i0;
} /* nVainqueur */

/* *********************************************************** */

void apprentissagePoids(int i0,neurone n){
	int i;
	float d;
	neurone tabTemp[NMAX];
	for(i=0;i<nbEntrees;i++){
		d=delta(i0,i);
		tabTemp[i].x=tab[i].x+(d*(n.x-tab[i].x));
		tabTemp[i].y=tab[i].y+(d*(n.y-tab[i].y));
	}
	for(i=0;i<nbEntrees;i++){
			tab[i].x=tabTemp[i].x;
			tab[i].y=tabTemp[i].y;
	}
	reduireEta();

} /* apprentissagePoids */

/* **************************************************** */

void unApprentissage(){
  int r;
	float temp;
     r=rand();
		 temp=((float)r/(float)RAND_MAX);
		 temp=temp*nbEntrees;
		 r=temp;//r = une ville au hasard
  apprentissagePoids(nVainqueur(entrees[r]),entrees[r]);
} /* unApprentissage */

/***************** main **************************/

int main(int argc,char **argv) {
int i,j;
temps=0;

/* initilisation de glut et creation de la fenetre */
glutInit(&argc,argv);
glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
glutInitWindowPosition(200,200);
glutInitWindowSize(HEIGHT,WIDTH);
glutCreateWindow("Kohonen");

/* initialisation OpenGL */
glClearColor(0,0,0,0);
glPointSize(2);

/* enregistrement des fonctions de rappel */
glutDisplayFunc(affichage);
glutKeyboardFunc(clavier);
glutReshapeFunc(reshape);
glutMouseFunc(mouse);

/* Entree dans la boucle principale */
glutMainLoop();
return 0;
} /* main */

/* **************************************************** */

void affichage() {
 int i,j,k,l;

 /* effacement de l'image avec la couleur de fond */
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 glPointSize(4);
 glLoadIdentity();

  if (etape==1){ // prise des coordonnees de l'entrees
    for(i=0;i<nbEntrees;i++){
        glBegin(GL_POINTS);
        glColor3f(1,0,0);
        glVertex2f(entrees[i].x,entrees[i].y);
        glEnd();
        }

    }
    
  else{ // une etape d'apprentissage
    glLoadIdentity();
      for(i=0;i<nbEntrees;i++) {//affichage Villes
        glBegin(GL_POINTS);
        glColor3f(1,0,0);
        glVertex2f(entrees[i].x,entrees[i].y);
        glEnd();
      }
    glColor3f(1,1,1);
    glBegin(GL_LINE_LOOP);
    for(i=0;i<nbEntrees;i++) {//affichage trajet
	     glVertex2f(tab[i].x,tab[i].y);
    }
    glEnd();
    glPointSize(3);
    glBegin(GL_POINTS);
    glColor3f(0,1,0);
    for(i=0;i<nbEntrees;i++) {//affichage Neurone
        glVertex2f(tab[i].x,tab[i].y);
    }
    glEnd();  
  }

  glFlush();
 /* On echange les buffers */
 glutSwapBuffers();
} /* affichage */


/* *************************************************** */

void reshape(int w,int h) {
  glViewport(0,0,w,h);
} /* reshape */

/* *************************************************** */

void clavier(unsigned char touche,int x,int y) {
 if (etape == 1) {//prendre coordonne
  if (touche == 'p') {
    initPoids();
    etape=2;
    glutPostRedisplay();
   }
 }
 else {//etape == 2
  if (touche == 'p') {
	 temps++;
   unApprentissage();
   glutPostRedisplay();
  }
  if (touche == 'q') {
    exit(0);
    }
 }

} /* clavier */


/* *************************************************** */

void mouse(int bouton,int etat,int x,int y) {
  float xf,yf;
  if(etape==1){
 /* si on appuie sur le bouton de gauche */
   if (bouton==GLUT_LEFT_BUTTON && etat ==GLUT_DOWN) {
     xf=((float)x-(HEIGHT/2))/(HEIGHT/2);
     yf=((WIDTH/2)-(float)y)/(WIDTH/2);
     entrees[nbEntrees].x=xf;   // sauvegarde de la position de la souris
     entrees[nbEntrees].y=yf;
     nbEntrees++;
     glutPostRedisplay();
   }
  }
} /* mouse */


/* *************************************************** */
/*
void fin_affiche_resultat() {
int i,j;

for(i=0;i<nbEntrees;i++) {
	printf("neurone[%d]= %f , %f",i,tab[i].x,tab[i].y);

}

} *//* fin_affiche_resultat */
